export default function taskManager(tasks: Array<() => Promise<void>>, maxConcurrentTasks: number): Promise<void> {
  return new Promise((resolve, reject) => {
    const remainingTasks = tasks;
    let currentlyRunningTasks = 0;

    const startNewTask = () => {
      if (remainingTasks.length === 0 && currentlyRunningTasks === 0) {
        return resolve();
      }

      const curTask = remainingTasks.pop() as () => Promise<void>;
      const curPromise = curTask();
      currentlyRunningTasks += 1;
      curPromise.then(() => {
        currentlyRunningTasks -= 1;
        return startNewTask();
      });
    };

    for (let i = 0; i < maxConcurrentTasks; i += 1) {
      startNewTask();
    }
  });
}
